@@ -31,10 +31,12 @@ import pymod2pkg
 import yaml
 
 from renderspec.distloader import RenderspecLoader
+from renderspec import utils
 from renderspec import versions
 
 
 # a variable that needs to be set for some functions in the context
+CONTEXT_VAR_PYPI_NAME = "pypi_name"
 CONTEXT_VAR_UPSTREAM_VERSION = "upstream_version"
 CONTEXT_VAR_RPM_RELEASE = "rpm_release"
 
@@ -46,6 +48,31 @@ def _context_check_variable(context, var_name, needed_by):
                                    " needed for '%s'" % (var_name, needed_by))
 
 
+def _context_upstream_version(context, pkg_version=None):
+    """return the version which should be set to the 'upstream_version'
+    variable in the jinja context"""
+    if pkg_version:
+        return pkg_version
+    else:
+        # try to auto-detect the version - for that we need the pypi name
+        _context_check_variable(context, CONTEXT_VAR_PYPI_NAME,
+                                'upstream_version')
+        pypi_name = context.vars[CONTEXT_VAR_PYPI_NAME]
+        # look for archives in the dir where the input template (.spec.j2)
+        # comes from. As fallback, also look in the current working dir
+        archives = utils._find_archives([context['input_template_dir'], '.'],
+                                        pypi_name)
+        for archive in archives:
+            with utils._extract_archive_to_tempdir(archive) as tmpdir:
+                pkg_info_file = utils._find_pkg_info(tmpdir)
+                if pkg_info_file:
+                    return utils._get_version_from_pkg_info(pkg_info_file)
+        # unable to autodetect the version
+        raise TemplateRuntimeError("Can not autodetect 'upstream_version' from"
+                                   " the following archives: '%s'" % (
+                                       ', '.join(archives)))
+
+
 def _context_py2rpmversion(context):
     """get a python PEP0440 compatible version and translate it to an RPM
     version"""
@@ -102,7 +129,12 @@ def _context_epoch(context, pkg_name):
     return context['epochs'].get(pkg_name, 0)
 
 
-def _context_py2name(context, pkg_name):
+def _context_py2name(context, pkg_name=None):
+    if not pkg_name:
+        # if the name is not given, try to get the name from the context
+        _context_check_variable(context, CONTEXT_VAR_PYPI_NAME,
+                                'py2name')
+        pkg_name = context.vars[CONTEXT_VAR_PYPI_NAME]
     return pymod2pkg.module2package(pkg_name, context['spec_style'])
 
 
@@ -181,6 +213,11 @@ def _globals_py2pkg(context, pkg_name, pkg_version=None):
     return _context_py2pkg(context, pkg_name, pkg_version)
 
 
+@contextfunction
+def _globals_upstream_version(context, pkg_version=None):
+    return _context_upstream_version(context, pkg_version)
+
+
 @contextfunction
 def _globals_py2rpmversion(context):
     return _context_py2rpmversion(context)
@@ -202,7 +239,7 @@ def _globals_license_spdx(context, value):
 
 
 @contextfunction
-def _globals_py2name(context, value):
+def _globals_py2name(context, value=None):
     return _context_py2name(context, value)
 
 
@@ -215,6 +252,7 @@ def _env_register_filters_and_globals(env):
     env.globals['py2name'] = _globals_py2name
     env.globals['epoch'] = _globals_epoch
     env.globals['license'] = _globals_license_spdx
+    env.globals['upstream_version'] = _globals_upstream_version
 
 
 def generate_spec(spec_style, epochs, requirements, input_template_path):
@@ -230,8 +268,10 @@ def generate_spec(spec_style, epochs, requirements, input_template_path):
     if spec_style in env.loader.list_templates():
         template_name = spec_style
     template = env.get_template(template_name)
+    input_template_dir = os.path.dirname(os.path.abspath(input_template_path))
     return template.render(spec_style=spec_style, epochs=epochs,
-                           requirements=requirements)
+                           requirements=requirements,
+                           input_template_dir=input_template_dir)
 
 
 def _is_fedora(distname):
